UNIT Aliens;
(* Implements the enemies. *)

INTERFACE

  CONST
  (* How many aliens on screen. *)
    NUM_ALIENS = 16;

(* Initializes the enemies. *)
  PROCEDURE Initialize;
(* Updates the enemies. *)
  PROCEDURE Update;
(* Draw enemies. *)
  PROCEDURE Draw;

IMPLEMENTATION

  USES
    Display, General, Particles, Player, Shots, Sprites,
    allegro5;

  TYPE
  (* Alien types. *)
    TAlienTypes = (
      atBug = 0,
      atArrow,
      atThiccboi,
      atN
    );

  (* Defines an alien. *)
    TAlien = RECORD
    (* Position. *)
      x, y: INTEGER;
    (* Type. *)
      AlienType: TAlienTypes;
    (* Statuses. *)
      ShotTimer, Blink, Life: INTEGER;
      Used: BOOLEAN;
    END;

  VAR
  (* The aliens. *)
    fAlien: ARRAY [1..NUM_ALIENS] OF TAlien;



(* Initializes the enemies. *)
  PROCEDURE Initialize;
  VAR
    Cnt: INTEGER;
  BEGIN
    FOR Cnt := 1 TO NUM_ALIENS DO
      fAlien[Cnt].Used := FALSE
  END;



(* Updates the enemies. *)
  PROCEDURE Update;
  VAR
    Ndx, NewQuota, NewX, cX, cY: INTEGER;
  BEGIN
  { Values for new enemies. }
    IF Frames MOD 120 <> 0 THEN
      NewQuota := 0
    ELSE
      NewQuota := IntegerBetween (2, 4);
    NewX := IntegerBetween (10, BUFFER_W - 50);
  { For each enemy... }
    FOR Ndx := 1 TO NUM_ALIENS DO
    BEGIN
      IF NOT fAlien[Ndx].Used THEN
      BEGIN
      { If this alien is unused, should it spawn? }
        IF NewQuota > 0 THEN
        BEGIN
          INC (NewX, IntegerBetween (40, 80));
          IF NewX > BUFFER_W - 60 THEN DEC (NewX, BUFFER_W - 60);

          fAlien[Ndx].x := NewX;

          fAlien[Ndx].y := IntegerBetween (-40, -30);
          fAlien[Ndx].AlienType := TAlienTypes (IntegerBetween (0, ORD (atN)));
          fAlien[Ndx].ShotTimer := IntegerBetween (1, 99);
          fAlien[Ndx].Blink := 0;
          fAlien[Ndx].Used := TRUE;

          CASE fAlien[Ndx].AlienType OF
          atBug:
            fAlien[Ndx].Life := 4;
          atArrow:
            fAlien[Ndx].Life := 2;
          atThiccboi:
            fAlien[Ndx].Life := 12;
          END;
          DEC (NewQuota)
        END
      END
      ELSE BEGIN
      { Updates enemy. }
        CASE fAlien[Ndx].AlienType OF
        atBug:
          IF Frames MOD 2 <> 0 THEN INC (fAlien[Ndx].y);
        atArrow:
          INC (fAlien[Ndx].y);
        atThiccboi:
          IF Frames MOD 4 = 0 THEN INC (fAlien[Ndx].y);
        END;

        IF fAlien[Ndx].y >= BUFFER_H THEN
        BEGIN
          fAlien[Ndx].Used := FALSE;
          CONTINUE
        END;

        IF fAlien[Ndx].Blink > 0 THEN DEC (fAlien[Ndx].Blink);
      { Collisions with shots. }
        IF Shots.Collide (
          FALSE,
          fAlien[Ndx].x, fAlien[Ndx].y,
          ALIEN_W[ORD (fAlien[Ndx].AlienType)], ALIEN_H[ORD (fAlien[Ndx].AlienType)]
        ) THEN
        BEGIN
          DEC (fAlien[Ndx].Life);
          fAlien[Ndx].Blink := 4
        END;
      { Get coordinates of enemy center. }
        cX := fAlien[Ndx].x + (ALIEN_W[ORD (fAlien[Ndx].AlienType)] DIV 2);
        cY := fAlien[Ndx].y + (ALIEN_H[ORD (fAlien[Ndx].AlienType)] DIV 2);
      { Test enemy life. }
        IF fAlien[Ndx].Life <= 0 THEN
        BEGIN
          Particles.Add (FALSE, cX, cY);

          CASE fAlien[Ndx].AlienType OF
          atBug:
            INC (Player.Score, 200);
          atArrow:
            INC (Player.Score, 150);
          atThiccboi:
            BEGIN
              INC (Player.Score, 800);

              Particles.Add (FALSE, cX - 10, cY -  4);
              Particles.Add (FALSE, cX +  4, cY + 10);
              Particles.Add (FALSE, cX +  8, cY +  8)
            END;
          END;
          fAlien[Ndx].Used := FALSE;
          CONTINUE
        END;
      { Check if shot. }
        DEC (fAlien[Ndx].ShotTimer);
        IF fAlien[Ndx].ShotTimer <= 0 THEN
          CASE fAlien[Ndx].AlienType OF
          atBug:
            BEGIN
              Shots.Add (FALSE, FALSE, cX, cY);
              fAlien[Ndx].ShotTimer := 150
            END;
          atArrow:
            BEGIN
              Shots.Add (FALSE, FALSE, cX, fAlien[Ndx].y);
              fAlien[Ndx].ShotTimer := 80
            END;
          atThiccboi:
            BEGIN
              Shots.Add (FALSE, TRUE, cX - 5, cY);
              Shots.Add (FALSE, TRUE, cX + 5, cY);
              Shots.Add (FALSE, TRUE, cX - 5, cY + 8);
              Shots.Add (FALSE, TRUE, cX + 5, cY + 8);
              fAlien[Ndx].ShotTimer := 200
            END;
          END
      END
    END
  END;



(* Draw enemies. *)
  PROCEDURE Draw;
  VAR
    Ndx: INTEGER;
  BEGIN
    FOR Ndx := 1 TO NUM_ALIENS DO
      IF (fAlien[Ndx].Used) AND (fAlien[Ndx].Blink <= 2) THEN
        al_draw_bitmap (
          Sprites.Alien[ORD (fAlien[Ndx].AlienType)],
          fAlien[Ndx].x, fAlien[Ndx].y,
          0
        )
  END;

END.
