UNIT Display;
(* Implements Display stuff. *)

INTERFACE

  USES
    allegro5;

  CONST
  (* Graphics buffer size. *)
    BUFFER_W = 320; BUFFER_H = 240;
  (* Display size. *)
    DISPLAY_SCALE = 3;
    DISPLAY_W = BUFFER_W * DISPLAY_SCALE; DISPLAY_H = BUFFER_H * DISPLAY_SCALE;


(* Initializes the display. *)
  PROCEDURE Initialize;
(* Prepares the output. *)
  PROCEDURE PreDraw;
(* Finalizes the output and swap buffers. *)
  PROCEDURE PostDraw;
(* Release resources. *)
  PROCEDURE Deinitialize;

(* Returns reference for the display event source. *)
  FUNCTION GetEventSource: ALLEGRO_EVENT_SOURCEptr;

IMPLEMENTATION

  USES
    General;

  VAR
    fDisplay: ALLEGRO_DISPLAYptr;
    fBuffer: ALLEGRO_BITMAPptr;

(* Initializes display. *)
  PROCEDURE Initialize;
  BEGIN
  { Create window. }
    al_set_new_display_option(ALLEGRO_SAMPLE_BUFFERS, 1, ALLEGRO_SUGGEST);
    al_set_new_display_option(ALLEGRO_SAMPLES, 8, ALLEGRO_SUGGEST);
    fDisplay := al_create_display (DISPLAY_W, DISPLAY_H);
    MustInit (fDisplay <> NIL, 'Display');

    fBuffer := al_create_bitmap (BUFFER_W, BUFFER_H);
    MustInit (fBuffer <> NIL, 'bitmap buffer')
  END;



(* Prepares the output. *)
  PROCEDURE PreDraw;
  BEGIN
    al_set_target_bitmap (fBuffer);
    al_clear_to_color (al_map_rgb (0,0,0))
  END;



(* Finalizes the output and swap buffers. *)
  PROCEDURE PostDraw;
  BEGIN
  { Scales backbuffer. }
    al_set_target_backbuffer (fDisplay);
    al_draw_scaled_bitmap (
      fBuffer,
      0, 0, BUFFER_W, BUFFER_H,
      0, 0, DISPLAY_W, DISPLAY_H,
      0
    );
  { Show frame. }
    al_flip_display
  END;



(* Release resources. *)
  PROCEDURE Deinitialize;
  BEGIN
    al_destroy_bitmap (fBuffer);
    al_destroy_display (fDisplay)
  END;



(* Returns reference for the display event source. *)
  FUNCTION GetEventSource: ALLEGRO_EVENT_SOURCEptr;
  BEGIN
    EXIT (al_get_display_event_source (fDisplay))
  END;

END.
