PROGRAM Game;
(* This is a complete copy of the source from Allegro Vivace's 'gameplay'
   section.

   URL: http://allegro-pas.sourceforge.net/tutorials/vivace/gameplay.php *)

USES
  Aliens, Audio, Display, General, HUD, Keyboard, Particles, Player,
  Shots, Sprites, Starfield,
  Allegro5, al5primitives;

VAR
  Timer: ALLEGRO_TIMERptr;
  Queue: ALLEGRO_EVENT_QUEUEptr;

(* Initializes the program. *)
  PROCEDURE Initialize;
  BEGIN
  { Initialize subsystems. }
    MustInit (al_init, 'Allegro');
    Keyboard.Initialize;
    Timer := al_create_timer (1 / 60);
    MustInit (Timer <> NIL, 'timer');
    Display.Initialize;
    MustInit (al_init_primitives_addon, 'primitives');
    Audio.Initialize;
    Sprites.Initialize;
    HUD.Initialize;
  { Event queue. }
    Queue := al_create_event_queue;
    MustInit (Queue <> NIL, 'queue');

    al_register_event_source (Queue, al_get_keyboard_event_source);
    al_register_event_source (Queue, Display.GetEventSource);
    al_register_event_source (Queue, al_get_timer_event_source (Timer));
  { Initialize random generator. }
    Randomize
  END;



(* Releases resources. *)
  PROCEDURE Deinitialize;
  BEGIN
    Sprites.Deinitialize;
    HUD.Deinitialize;
    Audio.Deinitialize;
    Display.Deinitialize;
    al_destroy_timer (Timer);
    al_destroy_event_queue (Queue)
  END;


VAR
  EndProgram, Redraw: BOOLEAN;
  Event: ALLEGRO_EVENT;
BEGIN
  Queue := NIL; TImer := NIL; { Avoids compilation warning. }
  Initialize;
{ Game initialization. }
  Particles.Initialize;
  Shots.Initialize;
  Player.Initialize;
  Aliens.Initialize;
  Starfield.initialize;

  Frames := 0;
  Score := 0;
{ Initialize loop. }
  EndProgram := FALSE;
  Redraw := TRUE;
  al_start_timer (Timer);
{ Game loop. }
  REPEAT
    al_wait_for_event (Queue, @Event);
    CASE Event.ftype OF
    ALLEGRO_EVENT_TIMER:
      BEGIN
      { Update stuff. }
        Particles.Update;
        Shots.Update;
        Starfield.Update;
        Player.Update;
        Aliens.Update;
        HUD.Update;
      { Exit? }
        IF Key[ALLEGRO_KEY_ESCAPE] <> 0 THEN EndProgram := TRUE;
      { Next frame. }
        Redraw := TRUE;
        INC (Frames)
      END;
    ALLEGRO_EVENT_DISPLAY_CLOSE:
      EndProgram := TRUE;
    END;
    IF NOT EndProgram THEN
    BEGIN
      Keyboard.HandleEvent (Event);
    { If must redraw and there's nothing pending... }
      IF Redraw AND al_is_event_queue_empty (Queue) THEN
      BEGIN
        Display.PreDraw;
          Starfield.Draw;
          Aliens.Draw;
          Shots.Draw;
          Particles.Draw;
          Player.Draw;
          HUD.Draw;
        Display.PostDraw;
        Redraw := FALSE
      END
    END
  UNTIL EndProgram;
  Deinitialize
END.
