UNIT HUD;
(* Implements the on screen score and remaining lives. *)

INTERFACE

(* Initializes HUD. *)
  PROCEDURE Initialize;
(* Updates HUD. *)
  PROCEDURE Update;
(* Draws HUD. *)
  PROCEDURE Draw;
(* Releases resources. *)
  PROCEDURE Deinitialize;

IMPLEMENTATION

  USES
    Display, General, Player, Sprites,
    allegro5, al5font;

  VAR
    fFont: ALLEGRO_FONTptr;
    fScore: LONGINT;

(* Initializes HUD. *)
  PROCEDURE Initialize;
  BEGIN
    fFont := al_create_builtin_font;
    MustInit (fFont <> NIL, 'font');

    fScore := 0
  END;



(* Updates HUD. *)
  PROCEDURE Update;
  VAR
    i, Diff: LONGINT;
  BEGIN
    IF Frames MOD 3 = 0 THEN
      FOR i := 5 DOWNTO 1 DO
      BEGIN
        Diff := 1 SHL i;
        IF fScore <= Player.Score - Diff THEN
          INC (fScore, Diff)
      END
  END;



(* Draws HUD. *)
  PROCEDURE Draw;
  CONST
    Spacing = LIFE_W + 1;
  VAR
    Cnt: INTEGER;
  BEGIN
    al_draw_textf (
      fFont, al_map_rgb_f (1,1,1),
      1, 1, 0,
      '%.06d', [fScore]
    );

    FOR Cnt := Player.NumLives - 1 DOWNTO 0 DO
      al_draw_bitmap (Sprites.Life, 1 + (Cnt * Spacing), 10, 0);

    IF Player.NumLives < 0 THEN
      al_draw_text (
        fFont, al_map_rgb_f (1, 1, 1),
        BUFFER_W DIV 2, BUFFER_H DIV 2, ALLEGRO_ALIGN_CENTER,
        'G A M E  O V E R'
    )
  END;



(* Releases resources. *)
  PROCEDURE Deinitialize;
  BEGIN
    al_destroy_font (fFont)
  END;

END.
