UNIT Particles;
(* Implements a particle system. *)

INTERFACE

  CONST
  (* How many particles. *)
    MAX_PARTICLES = 128;

(* (Re)initializes the particle system. *)
  PROCEDURE Initialize;
(* Adds a particle, if there's room. *)
  PROCEDURE Add (CONST aSpark: BOOLEAN; CONST aX, aY: INTEGER);
(* Updates particles. *)
  PROCEDURE Update;

(* Draws particles. *)
  PROCEDURE Draw;

IMPLEMENTATION

  USES
    Audio, General, Sprites,
    allegro5, al5audio;

  TYPE
  (* Stores particle. *)
    TParticle = RECORD
    (* Position. *)
      X, Y: INTEGER;
    (* Image. *)
      Frame: INTEGER;
    (* State. *)
      Spark, Used: BOOLEAN;
    END;

  VAR
  (* Particles. *)
    fParticle: ARRAY [1..MAX_PARTICLES] OF TParticle;

(* Initializes. *)
  PROCEDURE Initialize;
  VAR
    Cnt: INTEGER;
  BEGIN
    FOR Cnt := 1 TO MAX_PARTICLES DO fParticle[Cnt].Used := FALSE
  END;



(* Adds a particle. *)
  PROCEDURE Add (CONST aSpark: BOOLEAN; CONST aX, aY: INTEGER);
  VAR
    Cnt: INTEGER;
  BEGIN
  { Sound effect, if needed. }
    IF NOT aSpark THEN
      al_play_sample (
        SampleExplode[IntegerBetween (0, 2)],
        0.75, 0, 1, ALLEGRO_PLAYMODE_ONCE,
        NIL
      );
  { Add particle. }
    FOR Cnt := 1 TO MAX_PARTICLES DO
      IF NOT fParticle[Cnt].Used THEN
      BEGIN
        fParticle[Cnt].x := aX;
        fParticle[Cnt].y := aY;
        fParticle[Cnt].Frame := 0;
        fParticle[Cnt].Spark := aSpark;
        fParticle[Cnt].Used := TRUE;
        EXIT
      END
  END;



(* Updates particles. *)
  PROCEDURE Update;
  VAR
    Ndx: INTEGER;
  BEGIN
    FOR Ndx := 1 TO MAX_PARTICLES DO
      IF fParticle[Ndx].Used THEN
      BEGIN
        INC (fParticle[Ndx].Frame);
        IF (NOT fParticle[Ndx].Spark
            AND (fParticle[Ndx].Frame = EXPLOSION_FRAMES * 2))
        OR (    fParticle[Ndx].Spark
            AND (fParticle[Ndx].Frame = SPARK_FRAMES * 2))
        THEN
          fParticle[Ndx].Used := FALSE
      END
  END;



(* Draw particles. *)
  PROCEDURE Draw;
  VAR
    Ndx, lFrame, lX, lY: INTEGER;
    Bmp: ALLEGRO_BITMAPptr;
  BEGIN
    FOR Ndx := 1 TO MAX_PARTICLES DO
      IF fParticle[Ndx].Used THEN
      BEGIN
      { Get sprite number. }
        lFrame := fParticle[Ndx].Frame DIV 2;
        IF fParticle[Ndx].Spark THEN
          Bmp := Sprites.Sparks[lFrame]
        ELSE
          Bmp := Sprites.Explosion[lFrame];
      { Calculate position. }
        lX := fParticle[Ndx].x - (al_get_bitmap_width (Bmp) DIV 2);
        lY := fParticle[Ndx].y - (al_get_bitmap_height (Bmp) DIV 2);
      { Draw. }
        al_draw_bitmap (Bmp, lX, lY, 0)
      END
  END;

END.
