UNIT Player;
(* Implements player. *)

INTERFACE

  VAR
  (* Player score. *)
    Score: LONGINT;

(* Initializes ship. *)
  PROCEDURE Initialize;
(* Updates ship. *)
  PROCEDURE Update;
(* Draws ship. *)
  PROCEDURE Draw;

(* Returns number of lives. *)
  FUNCTION NumLives: INTEGER;

IMPLEMENTATION

  USES
    Display, Keyboard, Particles, Shots, Sprites,
    allegro5;

  CONST
    SPEED = 3;
    MAX_X = BUFFER_W - SHIP_W;
    MAX_Y = BUFFER_H - SHIP_H;

  TYPE
  (* Defines the player ship. *)
    TShip = RECORD
    (* Position. *)
      x, y: INTEGER;
    (* Lives. *)
      Lives: INTEGER;
    (* Statuses. *)
      ShotTimer, RespawnTimer, InvincibleTimer: INTEGER;
    END;

  VAR
    fShip: TShip;

(* Initializes. *)
  PROCEDURE Initialize;
  BEGIN
    fShip.x := (BUFFER_W DIV 2) - (SHIP_W DIV 2);
    fShip.y := (BUFFER_H DIV 2) - (SHIP_H DIV 2);
    fShip.ShotTimer := 0;
    fShip.Lives := 3;
    fShip.RespawnTimer := 0;
    fShip.InvincibleTimer := 120
  END;



(* Updates ship. *)
  PROCEDURE Update;
  VAR
    lX, lY: INTEGER;
  BEGIN
  { Check ship is ready. }
    IF fShip.Lives < 0 THEN EXIT;

    IF fShip.RespawnTimer > 0 THEN
    BEGIN
      DEC (fShip.RespawnTimer);
      EXIT
    END;
  { Move ship. }
    IF Key[ALLEGRO_KEY_LEFT]  <> 0 THEN DEC (fShip.x, SPEED);
    IF Key[ALLEGRO_KEY_RIGHT] <> 0 THEN INC (fShip.x, SPEED);
    IF Key[ALLEGRO_KEY_UP]    <> 0 THEN DEC (fShip.y, SPEED);
    IF Key[ALLEGRO_KEY_DOWN]  <> 0 THEN INC (fShip.y, SPEED);

    IF fShip.x < 0 THEN fShip.x := 0;
    IF fShip.y < 0 THEN fShip.y := 0;

    IF fShip.x > MAX_X THEN fShip.x := MAX_X;
    IF fShip.y > MAX_Y THEN fShip.y := MAX_Y;
  { Collisions. }
    IF fShip.InvincibleTimer > 0 THEN
      DEC (fShip.InvincibleTimer)
    ELSE if Shots.Collide (TRUE, fShip.x, fShip.y, SHIP_W, SHIP_H) THEN
    BEGIN
      lX := fShip.x + (SHIP_W DIV 2);
      lY := fShip.y + (SHIP_H DIV 2);
      Particles.Add (FALSE, lx    , ly    );
      Particles.Add (FALSE, lx + 4, ly + 2);
      Particles.Add (FALSE, lx - 2, ly - 4);
      Particles.Add (FALSE, lx + 1, ly - 5);

      DEC (fShip.Lives);
      fShip.RespawnTimer := 90;
      fShip.InvincibleTimer := 180
    END;
  { Shooting. }
    IF fShip.ShotTimer > 0 THEN
      DEC (fShip.ShotTimer)
    ELSE IF Key[ALLEGRO_KEY_X] <> 0 THEN
    BEGIN
      lX := fShip.x + (SHIP_W DIV 2);
      if Shots.Add (TRUE, FALSE, lX, fShip.y) THEN fShip.ShotTimer := 5
    END
  END;



(* Draws ship. *)
  PROCEDURE Draw;
  BEGIN
  { Check if draw or not. }
    IF (fShip.Lives < 0) OR (fShip.RespawnTimer > 0)
    OR (((fShip.InvincibleTimer DIV 2) MOD 3) = 1) { Blinking. }
    THEN
      EXIT;
  { Draw. }
    al_draw_bitmap (Sprites.Ship, fShip.x, fShip.y, 0)
  END;



(* Returns number of lives. *)
  FUNCTION NumLives: INTEGER;
  BEGIN
    EXIT (fShip.Lives)
  END;

END.
