UNIT Sprites;
(* Sprite loading and managing. *)

INTERFACE

  USES
    allegro5;

  CONST
  (* Sprite sizes. *)
    SHIP_W = 12; SHIP_H = 13;
    SHIP_SHOT_W = 2; SHIP_SHOT_H = 9;

    LIFE_W = 6; LIFE_H = 6;

    ALIEN_W: ARRAY [0..2] OF INTEGER = (14, 13, 45);
    ALIEN_H: ARRAY [0..2] OF INTEGER = (9, 10, 27);

    ALIEN_BUG_W      = 14; ALIEN_BUG_H    = 9;    { ALIEN[0] }
    ALIEN_ARROW_W    = 13; ALIEN_ARROW_H = 10;    { ALIEN[1] }
    ALIEN_THICCBOI_W = 45; ALIEN_THICCBOI_H = 27; { ALIEN[2] }

    ALIEN_SHOT_W = 4; ALIEN_SHOT_H = 4;

  (* Animation lengths. *)
    EXPLOSION_FRAMES = 4;
    SPARK_FRAMES = 3;

  VAR
  (* Reference to sprite graphics.  Use them as if they were read-only. *)
    Ship: ALLEGRO_BITMAPptr;
    ShipShot: ARRAY [0..1] OF ALLEGRO_BITMAPptr;
    Life: ALLEGRO_BITMAPptr;

    Alien: ARRAY [0..2] OF ALLEGRO_BITMAPptr;
    AlienShot: ALLEGRO_BITMAPptr;

    Explosion: ARRAY [0..EXPLOSION_FRAMES - 1] OF ALLEGRO_BITMAPptr;
    Sparks: ARRAY [0..SPARK_FRAMES - 1] OF ALLEGRO_BITMAPptr;

    PowerUp: ARRAY [0..3] OF ALLEGRO_BITMAPptr;

(* Loads and initializes sprite graphics. *)
  PROCEDURE Initialize;
(* Releases resources. *)
  PROCEDURE Deinitialize;
(* Tests if two rectangles collide. *)
  FUNCTION Collide (CONST ax1, ay1, ax2, ay2, bx1, by1, bx2, by2: INTEGER)
    : BOOLEAN;

IMPLEMENTATION

  USES
    General,
    al5image;

  VAR
    fSheet: ALLEGRO_BITMAPptr;

(* Loads and initializes sprite graphics. *)
  PROCEDURE Initialize;

    FUNCTION SpriteGrab (CONST x, y, w, h: INTEGER): ALLEGRO_BITMAPptr;
    BEGIN
      SpriteGrab := al_create_sub_bitmap (fSheet, x, y, w, h);
      MustInit (SpriteGrab <> NIL, 'sprite grab')
    END;

  BEGIN
    MustInit (al_init_image_addon, 'image');

    fSheet := al_load_bitmap ('spritesheet.png');
    MustInit (fSheet <> NIL, 'spritesheet');

    Ship := SpriteGrab (0, 0, SHIP_W, SHIP_H);

    Life := SpriteGrab (0, 14, LIFE_W, LIFE_H);

    ShipShot[0] := SpriteGrab (13, 0, SHIP_SHOT_W, SHIP_SHOT_H);
    ShipShot[1] := SpriteGrab (16, 0, SHIP_SHOT_W, SHIP_SHOT_H);

    Alien[0] := SpriteGrab (19,  0, ALIEN_BUG_W,      ALIEN_BUG_H);
    Alien[1] := SpriteGrab (19, 10, ALIEN_ARROW_W,    ALIEN_ARROW_H);
    Alien[2] := SpriteGrab ( 0, 21, ALIEN_THICCBOI_W, ALIEN_THICCBOI_H);

    AlienShot := SpriteGrab (13, 10, ALIEN_SHOT_W, ALIEN_SHOT_H);

    Explosion[0] := SpriteGrab (33, 10, 9, 9);
    Explosion[1] := SpriteGrab (43, 9, 11, 11);
    Explosion[2] := SpriteGrab (46, 21, 17, 18);
    Explosion[3] := SpriteGrab (46, 40, 17, 17);

    Sparks[0] := SpriteGrab (34, 0, 10, 8);
    Sparks[1] := SpriteGrab (45, 0, 7, 8);
    Sparks[2] := SpriteGrab (54, 0, 9, 8);

    Powerup[0] := SpriteGrab (0, 49, 9, 12);
    Powerup[1] := SpriteGrab (10, 49, 9, 12);
    Powerup[2] := SpriteGrab (20, 49, 9, 12);
    powerup[3] := SpriteGrab (30, 49, 9, 12)
  END;



(* Releases resources. *)
  PROCEDURE Deinitialize;

    PROCEDURE ReleaseArray (SprArray: ARRAY OF ALLEGRO_BITMAPptr);
    VAR
      Ndx: INTEGER;
    BEGIN
        FOR Ndx := LOW (SprArray) TO HIGH (SprArray) DO
          al_destroy_bitmap (SprArray[Ndx])
    END;

  BEGIN
    al_destroy_bitmap (Ship);
    ReleaseArray (ShipShot);
    ReleaseArray (Alien);
    al_destroy_bitmap (AlienShot);
    ReleaseArray (Explosion);
    ReleaseArray (Sparks);
    ReleaseArray (PowerUp);

    al_destroy_bitmap (fSheet)
  END;



(* Tests if two rectangles collide. *)
  FUNCTION Collide (CONST ax1, ay1, ax2, ay2, bx1, by1, bx2, by2: INTEGER)
    : BOOLEAN;
  BEGIN
    IF (ax1 > bx2) OR (ax2 < bx1) OR (ay1 > by2) OR (ay2 < by1) THEN
      EXIT (FALSE)
    ELSE
      EXIT (TRUE)
  END;

END.
